@@ -523,10 +523,30 @@ void alsa_set_volume(int l, int r)
 	if (!pcm_element)
 		return;
 
-	snd_mixer_selem_set_playback_volume(pcm_element,
-					    SND_MIXER_SCHN_FRONT_LEFT, l);
-	snd_mixer_selem_set_playback_volume(pcm_element,
-					    SND_MIXER_SCHN_FRONT_RIGHT, r);
+	if (snd_mixer_selem_is_playback_mono(pcm_element))
+	{
+		if (l > r)
+			snd_mixer_selem_set_playback_volume(pcm_element,
+							    SND_MIXER_SCHN_MONO, l);
+		else
+			snd_mixer_selem_set_playback_volume(pcm_element,
+							    SND_MIXER_SCHN_MONO, r);
+	}
+	else
+	{
+		snd_mixer_selem_set_playback_volume(pcm_element,
+						    SND_MIXER_SCHN_FRONT_LEFT, l);
+		snd_mixer_selem_set_playback_volume(pcm_element,
+						    SND_MIXER_SCHN_FRONT_RIGHT, r);
+	}
+
+	if (snd_mixer_selem_has_playback_switch(pcm_element) && !snd_mixer_selem_has_playback_switch_joined(pcm_element))
+	{
+		snd_mixer_selem_set_playback_switch(pcm_element,
+			SND_MIXER_SCHN_FRONT_LEFT, l != 0);
+		snd_mixer_selem_set_playback_switch(pcm_element,
+			SND_MIXER_SCHN_FRONT_RIGHT, r != 0);
+	}
 }
 
 
@@ -804,15 +824,13 @@ static void alsa_write_out_thread_data(void)
 static void *alsa_loop(void *arg)
 {
 	int npfds = snd_pcm_poll_descriptors_count(alsa_pcm);
-	struct pollfd *pfds;
-	unsigned short *revents;
+	int wr = 0;
 
 	g_mutex_lock(alsa_mutex);
 
 	if (npfds <= 0)
 		goto _error;
-	pfds = alloca(sizeof(*pfds) * npfds);
-	revents = alloca(sizeof(*revents) * npfds);
+
 	while (going && alsa_pcm)
 	{
 		if (get_thread_buffer_filled() > prebuffer_size)
@@ -820,23 +838,14 @@ static void *alsa_loop(void *arg)
 		if (!paused && !prebuffer &&
 		    get_thread_buffer_filled() > hw_period_size_in)
 		{
-			snd_pcm_poll_descriptors(alsa_pcm, pfds, npfds);
-			if (poll(pfds, npfds, 10) > 0)
+			wr = snd_pcm_wait(alsa_pcm, 10);
+			if (wr > 0)
 			{
-				/*
-				 * need to check revents.  poll() with
-				 * dmix returns a postive value even
-				 * if no data is available
-				 */
-				int i;
-				snd_pcm_poll_descriptors_revents(alsa_pcm, pfds,
-								 npfds, revents);
-				for (i = 0; i < npfds; i++)
-					if (revents[i] & POLLOUT)
-					{
-						alsa_write_out_thread_data();
-						break;
-					}
+				alsa_write_out_thread_data();
+			}
+			else if (wr < 0)
+			{
+				alsa_handle_error(wr);
 			}
 		}
 		else
